import Cache = require('./cache');
declare const _default: {
    new (): {
        MODULE_ENTRY: Cache;
        PATH: Cache;
        REAL_FILE_PATH: Cache;
        REAL_DIRECTORY_PATH: Cache;
    };
};
export = _default;
